/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 *********************************************************************/

package net.sf.jcmdlineparser.options;

import java.util.Set;

/**
 * 
 * 
 * Helper class for {@link AbstractOption} operations.
 * 
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-11-23
 * 
 */
public class OptionUtils {

	/**
	 * 
	 */
	public final static String shortPrefix = "-";

	/**
	 * 
	 */
	public final static String longPrefix = shortPrefix + shortPrefix;

	private OptionUtils() {

	}

	/**
	 * 
	 * Checks whether option ident string matches identifier of one of given
	 * options.
	 * 
	 * @param options
	 *            options that are checked for given ident string
	 * @param ident
	 *            ident string that is looked for
	 * @return true, if given ident string matches identifier of one of given
	 *         options; false otherwise
	 */
	public static boolean optionKnown(Set<AbstractOption<?>> options,
			String ident) {
		ident = ident.replaceAll(shortPrefix, "");
		for (AbstractOption<?> o : options) {
			if (o.getKeyLong().equals(ident)
					|| (ident.length() == 1 && o.getKeyShort() == ident
							.charAt(0)))
				return true;
		}
		return false;
	}

	/**
	 * 
	 * Finds option which ident string matches given one.
	 * 
	 * @param options
	 *            options that are searched for given ident string
	 * @param ident
	 *            ident string that is searched for
	 * @return option which ident string matches given one
	 * @throws NoSuchOptionException
	 *             if no such option could be found
	 */
	public static AbstractOption<?> getOptionForKey(
			Set<AbstractOption<?>> options, String ident)
			throws NoSuchOptionException {

		ident = ident.replaceAll(shortPrefix, "");

		if (ident == null || ident.length() < 1)
			throw new NullPointerException("ident=" + ident);

		if (ident.length() == 1)
			return getOptionForShortKey(options, ident.charAt(0));

		return getOptionForLongKey(options, ident);
	}

	private static AbstractOption<?> getOptionForShortKey(
			Set<AbstractOption<?>> options, char c)
			throws NoSuchOptionException {
		for (AbstractOption<?> o : options) {
			if (o.getKeyShort() == c)
				return o;
		}
		throw new NoSuchOptionException("option \"" + c + "\" not known");
	}

	private static AbstractOption<?> getOptionForLongKey(
			Set<AbstractOption<?>> options, String ident)
			throws NoSuchOptionException {
		for (AbstractOption<?> o : options) {
			if (o.getKeyLong().equals(ident))
				return o;
		}
		throw new NoSuchOptionException("option \"" + ident + "\" not known");
	}

	/**
	 * 
	 * Checks given set of options for option clashing. Throws an exception if
	 * there are at least two options that cannot be set together. Does nothing
	 * if all given options may coexist.
	 * 
	 * @param options
	 *            options that are checked for option clashing
	 * @throws OptionClashException
	 *             if there are at least two options that cannot coexist
	 */
	public static void checkOptionClashes(Set<AbstractOption<?>> options)
			throws OptionClashException {

		for (AbstractOption<?> o1 : options) {

			for (AbstractOption<?> o2 : options) {

				if (o1.isSet() && o2.isSet()
						&& o1.getClashOptions().contains(o2))
					throw new OptionClashException("options \"" + o1
							+ "\" and \"" + o2 + "\" cannot coexist!");
			}
		}
	}

	/**
	 * 
	 * Checks given options for missing dependencies. If given set of options
	 * contains one option that requires another option which is not contained
	 * in this options set, throws an {@code MissingRequiredOptionException}.
	 * Does also throw an exception, if required option is contained in given
	 * options set but is not set. Does nothing if all options in given set only
	 * require options that are also contained in given set and all required
	 * options are set.
	 * 
	 * @param options
	 *            options that are checked for dependencies
	 * @throws MissingRequiredOptionException
	 *             if there is one or more option which is missing
	 */
	public static void checkRequiredDependency(Set<AbstractOption<?>> options)
			throws MissingRequiredOptionException {

		for (AbstractOption<?> o1 : options) {

			Set<AbstractOption<?>> req = o1.getRequiredOptions();
			for (AbstractOption<?> o2 : req) {

				// check only requirements if option itself is set
				if (o1.isSet()) {

					if (o2.isSet()) {
						// all good
					} else
						throw new MissingRequiredOptionException("option \""
								+ o1 + "\" requires option \"" + o2
								+ "\" which is not set!");
				}
			}
		}
	}

	/**
	 * 
	 * Checks if all options which are required are also set.
	 * 
	 * @param options options that are checked
	 * @throws MissingRequiredOptionException if one or more options are required but not set
	 */
	public static void checkRequiredSimple(Set<AbstractOption<?>> options)
			throws MissingRequiredOptionException {
		
		for (AbstractOption<?> o1 : options) {
			
			if (o1.isRequired()) {
				if (o1.isSet()) {
					// all good
				} else
					throw new MissingRequiredOptionException("option \"" + o1
							+ "\" is required");
			}
		}
	}
}
