/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 *********************************************************************/

package net.sf.jcmdlineparser.parser;

import java.io.OutputStream;

import net.sf.jcmdlineparser.ParserException;
import net.sf.jcmdlineparser.options.AbstractOption;
import net.sf.jcmdlineparser.options.DuplicateOptionIdentifierException;
import net.sf.jcmdlineparser.strategy.Strategy;


/**
 * <p>
 * An implementation of <code>CmdLineParser</code> will provide command line
 * argument parsing functionality.
 * </p>
 * <p>
 * Parsing logic is delegated to given <code>Strategy</code>s
 * </p>
 * 
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-08-10
 * @see Strategy
 * 
 * 
 */
public interface CmdLineParser {

	/**
	 * <p>
	 * Adds a command line option that will be parsed for.
	 * </p>
	 * 
	 * @param <O>
	 *            Implementation type of Option. Could be an <code>String</code>
	 *            option or <code>Integer</code> option, for instance.
	 * @param o
	 *            {@code AbstractOption} that is parsed for.
	 * @throws DuplicateOptionIdentifierException
	 *             if provided option is already registered
	 * @see AbstractOption
	 */
	<O> void registerOption(AbstractOption<O> o)
			throws DuplicateOptionIdentifierException;

	/**
	 * <p>
	 * Add a (custom) <code>Strategy</code> for parsing. Most common parsing
	 * strategies are included by default, such as parsing strategies for
	 * expressions like this: <br>
	 * <code>--key=value
	 * <br>
	 * --key value1 [value2] [value3] [..]
	 * <br>
	 * -v1[v2][v3][..]
	 * </code> <br>
	 * Therefore, in most cases, there will be no need to add a further
	 * strategy.
	 * </p>
	 * 
	 * @param s
	 *            <code>Strategy</code> for parsing.
	 * @see Strategy
	 */
	<S extends Strategy> void addStrategy(S s);

	/**
	 * <p>
	 * Removes all registered parsing strategies from this
	 * <code>CmdLineParser</code>.
	 * </p>
	 * 
	 * @see Strategy
	 */
	void removeAllStrategies();

	/**
	 * <p>
	 * Parses command line arguments. All registered strategies will be used to
	 * support different command line argument formating.
	 * </p>
	 * 
	 * @param args
	 *            Array of <code>String</code> that is parsed.
	 * @return Remaining command line arguments, that could not be recognized.
	 * @throws ParserException
	 */
	String[] parse(String[] args) throws ParserException;

	/**
	 * <p>
	 * Parses command line arguments. All registered strategies will be used, to
	 * support different command line argument formating.
	 * </p>
	 * 
	 * @param args
	 *            Single string that is parsed.
	 * @return Remaining command line arguments, that could not be recognized.
	 * @throws ParserException
	 */
	String parse(String args) throws ParserException;

	/**
	 * <p>
	 * Prints a formatted help text to STDOUT.
	 * </p>
	 */
	void printHelp(String usageString);

	/**
	 * <p>
	 * Prints a formatted help text to <code>out<code>.
	 * </p>
	 * 
	 * @param out
	 *            OutputStream help text is printed to.
	 */
	void printHelp(String usageString, OutputStream out);

}
