/**********************************************************************
Copyright (c) 2009-2010 Alexander Kerner. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
 *********************************************************************/

package net.sf.jcmdlineparser.parser;

import java.io.OutputStream;
import java.util.Arrays;
import java.util.LinkedHashSet;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.sf.jcmdlineparser.ParserException;
import net.sf.jcmdlineparser.helpprinter.SimpleHelpPrinter;
import net.sf.jcmdlineparser.options.AbstractOption;
import net.sf.jcmdlineparser.options.DuplicateOptionIdentifierException;
import net.sf.jcmdlineparser.options.MissingRequiredOptionException;
import net.sf.jcmdlineparser.options.NoOptionsGivenException;
import net.sf.jcmdlineparser.options.OptionClashException;
import net.sf.jcmdlineparser.options.OptionUtils;
import net.sf.jcmdlineparser.strategy.RulesGeneral;
import net.sf.jcmdlineparser.strategy.Strategy;
import net.sf.jcmdlineparser.strategy.StrategyImpl;


/**
 * <p>
 * Implementation for {@link CmdLineParser}.
 * </p>
 * 
 * @see Strategy
 * @author <a href="mailto:alex.kerner.24@googlemail.com">Alexander Kerner</a>
 * @version 2010-10-19
 * 
 */
public class CmdLineParserImpl implements CmdLineParser {

	private final static Logger log = LoggerFactory.getLogger(CmdLineParserImpl.class);

	private final LinkedHashSet<Strategy> strat = new LinkedHashSet<Strategy>();
	private final LinkedHashSet<AbstractOption<?>> options = new LinkedHashSet<AbstractOption<?>>();

	// Constructor //

	/**
	 * 
	 * TODO
	 * 
	 */
	public CmdLineParserImpl() {
//		strat.add(new SimpleStrategy());
//		strat.add(new SpaceSeparatedStrategy());
		synchronized(strat){
		strat.add(new StrategyImpl(new RulesGeneral()));
		}
	}

	// Private //

	private void checkOptions() throws OptionClashException,
			MissingRequiredOptionException {
		synchronized(options){
		OptionUtils.checkOptionClashes(options);
		OptionUtils.checkRequiredSimple(options);
		OptionUtils.checkRequiredDependency(options);
		}
	}

	// Implement //

	public synchronized <S extends Strategy> void addStrategy(S s) {
		if(s == null)
			throw new NullPointerException();
		strat.add(s);
	}

	public synchronized String[] parse(String[] args) throws ParserException {
		if (args == null || args.length == 0)
			throw new NoOptionsGivenException("No commandline options given");
		log.debug("Parsing args: \"" + Arrays.toString(args)
				+ "\", Strategies: \"" + strat + "\"");
		for (Strategy s : strat)
			args = s.parse(args, this.options);
		checkOptions();
		return args;
	}

	public synchronized String parse(String args) throws ParserException {
		if (args == null || args.length() == 0)
			throw new NoOptionsGivenException("No commandline options given");
		log.debug("Parsing args: \"" + args + "\", Strategies: \"" + strat
				+ "\"");
		for (Strategy s : strat)
			args = s.parse(args, this.options);
		checkOptions();
		return args;
	}

	public synchronized <O> void registerOption(AbstractOption<O> o)
			throws DuplicateOptionIdentifierException {
		if(o == null)
			throw new NullPointerException();
		
		// TODO stupid workaround since options.contains() depends on hashcode which does not reflect equality correct.
		for(AbstractOption<?> oo : options){
			if(oo.equals(o))
				throw new DuplicateOptionIdentifierException("option " + o
						+ " already registered");
		}
//		if (options.contains(o))
//			throw new DuplicateOptionIdentifierException("option " + o
//					+ " already registered");
		options.add(o);
	}

	public synchronized void removeAllStrategies() {
		strat.clear();
	}

	public void printHelp(String usageString) {
		new SimpleHelpPrinter().printHelp(options, usageString);
	}

	public void printHelp(String usageString, OutputStream out) {
		new SimpleHelpPrinter().printHelp(options, usageString, out);
	}

}
